--
-- MixerWagon
-- Class for all sprayers
--
-- @author  Stefan Geiger
-- @date  24/02/08
--
-- Copyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.
-- Edit: by Farmer_Andy for pigForage

source("dataS/scripts/vehicles/specializations/SetTurnedOnEvent.lua");

MixerWagon = {};
local MixerWagon_directory = g_currentModDirectory;


function MixerWagon.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Fillable, specializations) and SpecializationUtil.hasSpecialization(TurnOnVehicle, specializations);
end;

function MixerWagon:load(xmlFile)
    self.mixerWagonSuperAllowFillType = self.allowFillType;
    self.allowFillType = Utils.overwrittenFunction(self.allowFillType, MixerWagon.allowFillType);
    self.getFillLevel = Utils.overwrittenFunction(self.getFillLevel, MixerWagon.getFillLevel);
    self.setFillLevel = Utils.overwrittenFunction(self.setFillLevel, MixerWagon.setFillLevel);
    self.resetFillLevelIfNeeded = Utils.overwrittenFunction(self.resetFillLevelIfNeeded, MixerWagon.resetFillLevelIfNeeded);
    self.fillShovelFromGroundValue = Utils.overwrittenFunction(self.fillShovelFromGroundValue, MixerWagon.fillShovelFromGroundValue);
    self.fillShovelFromTrigger = Utils.overwrittenFunction(self.fillShovelFromTrigger, MixerWagon.fillShovelFromTrigger);
    -- self.mixerWagonBaleTriggerCallback = MixerWagon.mixerWagonBaleTriggerCallback;
    self.getCanBeTurnedOn = Utils.overwrittenFunction(self.getCanBeTurnedOn, MixerWagon.getCanBeTurnedOn);
    self.getIsTurnedOnAllowed = Utils.overwrittenFunction(self.getIsTurnedOnAllowed, MixerWagon.getIsTurnedOnAllowed);
    self.drawMixerWagonHelp = SpecializationUtil.callSpecializationsFunction("drawMixerWagonHelp");
    self.getAllowFillShovel = Utils.overwrittenFunction(self.getAllowFillShovel, MixerWagon.getAllowFillShovel);
    self.mixerWagonNeedsTurnOn = true;
    if self.shovelNodes ~= nil then
        self.mixerWagonNeedsTurnOn = true;
    end
    self.mixerWagonPickupSoundEnabled = false;
    if self.isClient then
        self.sampleMixerWagonPickup = Utils.loadSample(xmlFile, {}, "vehicle.mixerWagonPickupSound", nil, self.baseDirectory, self.components[1].node);
        self.sampleMixerWagonStart = Utils.loadSample(xmlFile, {}, "vehicle.mixerWagonPickupStartSound", nil, self.baseDirectory);
        self.sampleMixerWagonStop = Utils.loadSample(xmlFile, {}, "vehicle.mixerWagonPickupStopSound", nil, self.baseDirectory);
        self.mixerWagonPickupSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.mixerWagonPickupSound#pitchOffset"), 1);
        self.mixerWagonPickupSoundPickupPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.mixerWagonPickupSound#pickupPitchOffset"), 0.8*self.mixerWagonPickupSoundPitchOffset);
        self.mixerWagonMixRotatingParts = Utils.loadRotationNodes(xmlFile, {}, "vehicle.mixerWagonRotatingParts.mixerWagonRotatingPart", "mixerWagonMix", self.components);
        self.mixerWagonPickupRotatingParts = Utils.loadRotationNodes(xmlFile, {}, "vehicle.mixerWagonRotatingParts.mixerWagonRotatingPart", "mixerWagonPickup", self.components);
    end
    self.mixerWagonFillTypes = {};
    self.fillTypeToMixerWagonFillType = {};
    local i=0;
    while true do
        local baseName = string.format("vehicle.mixerWagonFillTypes.mixerWagonFillType(%d)", i);
        if not hasXMLProperty(xmlFile, baseName) then
            break;
        end;
        local entry = {};
        entry.fillTypes = {};
        local j=0;
        while true do
            local fillTypeKey = baseName..string.format(".fillType(%d)", j);
            if not hasXMLProperty(xmlFile, fillTypeKey) then
                break;
            end;
            local fillType = getXMLString(xmlFile, fillTypeKey.."#fillType");
            if fillType ~= nil then
                local fillTypeInt = Fillable.fillTypeNameToInt[fillType];
                if fillTypeInt ~= nil then
                    if self.fillTypeToMixerWagonFillType[fillTypeInt] == nil then
                        entry.fillTypes[fillTypeInt] = true;
                        self.fillTypeToMixerWagonFillType[fillTypeInt] = entry;
                    else
                        print("Warning: Invalid mixerWagonFillType '"..fillType.."' used multiple times in '"..self.configFileName.."'.");
                    end
                else
                    print("Warning: Invalid mixerWagonFillType '"..fillType.."' in '"..self.configFileName.. "'.");
                end
            end
            j = j+1;
        end
        entry.minPercentage = Utils.getNoNil(getXMLFloat(xmlFile, baseName.."#minPercentage"), 0)*0.01;
        entry.maxPercentage = Utils.getNoNil(getXMLFloat(xmlFile, baseName.."#maxPercentage"), 100)*0.01;
        entry.fillLevel = 0;
        local name = Utils.getNoNil(getXMLString(xmlFile, baseName.."#name"), "unknown");
        entry.name = name;
        local i18n = g_i18n;
        if self.customEnvironment ~= nil then
            i18n = _G[self.customEnvironment].g_i18n;
        end;
        entry.nameI18N = i18n:getText(name);
        if next(entry.fillTypes) ~= nil then
            local width, height = getNormalizedScreenValues(350*g_currentMission.vehicleHudScale, 3*g_currentMission.vehicleHudScale)
            --entry.statusBarColor = {0.0232, 0.4508, 0.8388, 1};
            --entry.statusBarColor2 = {1, 0, 0, 1}
			entry.statusBarColor = {0.459, 0.702, 0.298, 1};
            entry.statusBarColor2 = {1, 0, 0, 1}
			entry.statusBar = StatusBar:new("dataS2/menu/white.png", nil, entry.statusBarColor, {0.04, 0.04, 0.04, 1}, nil, 0, 0, width, height);
            table.insert(self.mixerWagonFillTypes, entry);
        end
        i = i+1;
    end;
    if self.isServer then
        -- self.mixerWagonBaleTriggerId = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.mixerWagonBaleTrigger#index"));
        -- if self.mixerWagonBaleTriggerId ~= nil then
            -- addTrigger(self.mixerWagonBaleTriggerId, "mixerWagonBaleTriggerCallback", self);
        -- end
    end
    self.mixingActiveTimerMax = 5000;
    self.mixingActiveTimer = 0;
    fillTypeOverlayWidth, fillTypeOverlayHeight = getNormalizedScreenValues(7*g_currentMission.vehicleHudScale, 30*g_currentMission.vehicleHudScale);
    _, self.fillTypeOverlayLineSpacing = getNormalizedScreenValues(0, 35*g_currentMission.vehicleHudScale);
	
	self.mixerWagonFillTypeRangeMarkerLeftOverlay = Overlay:new("mixerWagonFillTypeRangeMarkerLeftOverlay", Utils.getFilename("scripts/mixerWagonHUD_rangeMarkerLeft.png", MixerWagon_directory), 0, 0, fillTypeOverlayWidth, fillTypeOverlayHeight);
    self.mixerWagonFillTypeRangeMarkerRightOverlay = Overlay:new("mixerWagonFillTypeRangeMarkerRightOverlay", Utils.getFilename("scripts/mixerWagonHUD_rangeMarkerLeft.png", MixerWagon_directory), 0, 0, fillTypeOverlayWidth, fillTypeOverlayHeight);
	
	--self.mixerWagonFillTypeRangeMarkerLeftOverlay = Overlay:new("", "dataS2/menu/hud/mixerWagonHUD_rangeMarkerLeft.png", 0, 0, fillTypeOverlayWidth, fillTypeOverlayHeight);
	--self.mixerWagonFillTypeRangeMarkerRightOverlay = Overlay:new("", "dataS2/menu/hud/mixerWagonHUD_rangeMarkerLeft.png", 0, 0, fillTypeOverlayWidth, fillTypeOverlayHeight);
	
	self.mixerWagonFillTypeRangeMarkerRightOverlay:setInvertX(true);
    _, self.mixerFillLevelTextOffsetY = getNormalizedScreenValues(0, 5*g_currentMission.vehicleHudScale);
    _, self.mixerStatusbarOffsetY = getNormalizedScreenValues(0, 14*g_currentMission.vehicleHudScale);
    self.mixerWagonLastPickupTime = -10000;
    -- disable fill level sync and save of Fillable
    self.synchronizeFillLevel = false;
    self.mixerWagonDirtyFlag = self:getNextDirtyFlag();
end;

function MixerWagon:delete()
    if self.isServer then
        -- if self.mixerWagonBaleTriggerId ~= nil then
            -- removeTrigger(self.mixerWagonBaleTriggerId);
        -- end
    end
    for _,entry in pairs(self.mixerWagonFillTypes) do
        entry.statusBar:delete();
    end
    if self.isClient then
        Utils.deleteSample(self.sampleMixerWagonPickup);
        Utils.deleteSample(self.sampleMixerWagonStart);
        Utils.deleteSample(self.sampleMixerWagonStop);
    end;
    self.mixerWagonFillTypeRangeMarkerLeftOverlay:delete();
    self.mixerWagonFillTypeRangeMarkerRightOverlay:delete();
end;

function MixerWagon:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
    self.fillLevel = 0;
    for i=1, table.getn(self.mixerWagonFillTypes) do
        local fillTypeKey = key..string.format(".mixerWagonFillType%d#fillLevel", i);
        local fillLevel = getXMLFloat(xmlFile, fillTypeKey);
        if fillLevel == nil then
            fillLevel = 0;
        end
        self:setFillLevel(fillLevel, next(self.mixerWagonFillTypes[i].fillTypes));
    end
    return BaseMission.VEHICLE_LOAD_OK;
end;

function MixerWagon:getSaveAttributesAndNodes(nodeIdent)
    local nodes = "";
    for i=1, table.getn(self.mixerWagonFillTypes) do
        if i>1 then
            nodes = nodes.."\n";
        end;
        local fillLevel = self.mixerWagonFillTypes[i].fillLevel;
        nodes = nodes.. nodeIdent..string.format('<mixerWagonFillType%d', i)..' fillLevel="'..fillLevel..'" />';
    end;
    return "", nodes;
end;

function MixerWagon:readStream(streamId, connection)
    self.fillLevel = 0;
    for i=1,table.getn(self.mixerWagonFillTypes) do
        local fillLevel = streamReadFloat32(streamId);
        self:setFillLevel(fillLevel, next(self.mixerWagonFillTypes[i].fillTypes));
    end
end

function MixerWagon:writeStream(streamId, connection)
    for i=1,table.getn(self.mixerWagonFillTypes) do
        streamWriteFloat32(streamId, self.mixerWagonFillTypes[i].fillLevel);
    end
end

function MixerWagon:readUpdateStream(streamId, timestamp, connection)
    if connection:getIsServer() then
        if streamReadBool(streamId) then
            self.mixingActiveTimer = streamReadUIntN(streamId, 4)/15 * self.mixingActiveTimerMax;
            for i=1,table.getn(self.mixerWagonFillTypes) do
                local fillLevel = streamReadUInt8(streamId)/255*self:getCapacity();
                self:setFillLevel(fillLevel, next(self.mixerWagonFillTypes[i].fillTypes));
            end
        end;
    end
end

function MixerWagon:writeUpdateStream(streamId, connection, dirtyMask)
    if not connection:getIsServer() then
        if streamWriteBool(streamId, bitAND(dirtyMask, self.mixerWagonDirtyFlag) ~= 0) then
            streamWriteUIntN(streamId, math.floor(self.mixingActiveTimer/self.mixingActiveTimerMax*15), 4);
            for i=1,table.getn(self.mixerWagonFillTypes) do
                local percent = 0;
                if self:getCapacity() ~= 0 then
                    percent = Utils.clamp(self.mixerWagonFillTypes[i].fillLevel / self:getCapacity(), 0, 1);
                end;
                streamWriteUInt8(streamId, math.floor(percent*255));
            end
        end
    end
end;

function MixerWagon:mouseEvent(posX, posY, isDown, isUp, button)
end;

function MixerWagon:keyEvent(unicode, sym, modifier, isDown)
end;

function MixerWagon:update(dt)
    if self.isClient then
        if self:getIsActive() then
            if self.mixingActiveTimer > 0 or self:getIsTurnedOn() or self.tipState == Trailer.TIPSTATE_OPENING or self.tipState == Trailer.TIPSTATE_OPEN then
                if self:getIsActiveForSound() then
                    if not Utils.isSamplePlaying(self.sampleMixerWagonStart, 1.8*dt) then
                        Utils.playSample(self.sampleMixerWagonPickup, 0, 0, nil);
                    end
                else
                    Utils.play3DSample(self.sampleMixerWagonPickup);
                end
                if Utils.isSamplePlaying(self.sampleMixerWagonPickup) then
                    if self.mixerWagonLastPickupTime >= g_currentMission.time-200 then
                        Utils.setSamplePitch(self.sampleMixerWagonPickup, self.mixerWagonPickupSoundPickupPitchOffset);
                    else
                        Utils.setSamplePitch(self.sampleMixerWagonPickup, self.mixerWagonPickupSoundPitchOffset);
                    end;
                end
            else
                if self:getIsActiveForSound() then
                    Utils.stopSample(self.sampleMixerWagonPickup);
                else
                    Utils.stop3DSample(self.sampleMixerWagonPickup);
                end;
            end
        end
        -- update pickup rotating parts
        Utils.updateRotationNodes(self, self.mixerWagonPickupRotatingParts, dt, self:getIsActive() and self:getIsTurnedOn());
        Utils.updateRotationNodes(self, self.mixerWagonMixRotatingParts, dt, self.mixingActiveTimer > 0 or self:getIsTurnedOn() or self.tipState == Trailer.TIPSTATE_OPENING or self.tipState == Trailer.TIPSTATE_OPEN);
    end;
    if self.mixingActiveTimer > 0 or self:getIsTurnedOn() or self.tipState == Trailer.TIPSTATE_OPENING or self.tipState == Trailer.TIPSTATE_OPEN then
        self.mixingActiveTimer = self.mixingActiveTimer - dt;
    end
end;

function MixerWagon:updateTick(dt)
end;

function MixerWagon:draw()
    if self:getIsActiveForInput(true) then
        local numFillTypes = table.getn(self.mixerWagonFillTypes);
        if numFillTypes > 0 then
            local helpHeight = g_currentMission.hudHelpTextSize + 2*g_currentMission.hudHelpTextLineSpacing;
            helpHeight = helpHeight + numFillTypes*self.fillTypeOverlayLineSpacing;
            g_currentMission:addHelpTextFunction(self.drawMixerWagonHelp, self, helpHeight, nil);
        end
    end
end;

function MixerWagon:onDetach()
end;

function MixerWagon:onLeave()
end;

function MixerWagon:onDeactivateSounds()
    if self.isClient then
        Utils.stopSample(self.sampleMixerWagonPickup, true);
        Utils.stopSample(self.sampleMixerWagonStart, true);
        Utils.stop3DSample(self.sampleMixerWagonPickup);
    end;
end;

-- overwrite Fillable.resetFillLevelIfNeeded
function MixerWagon:resetFillLevelIfNeeded(superFunc, fillType)
    local mixerWagonFillType = self.fillTypeToMixerWagonFillType[fillType];
    if mixerWagonFillType ~= nil then
        -- nothing to do
        return;
    end
    superFunc(self, fillType);
end;

function MixerWagon:allowFillType(superFunc, fillType, allowEmptying)
    local mixerWagonFillType = self.fillTypeToMixerWagonFillType[fillType];
    if mixerWagonFillType ~= nil then
        return true;
    end
    return superFunc(self, fillType, allowEmptying);
end

function MixerWagon:getFillLevel(superFunc, fillType)
    local mixerWagonFillType = self.fillTypeToMixerWagonFillType[fillType];
    if mixerWagonFillType ~= nil then
        return mixerWagonFillType.fillLevel;
    end
    return superFunc(self, fillType);
end

function MixerWagon:setFillLevel(superFunc, fillLevel, fillType, force, fillSourceStruct)
    local mixerWagonFillType = self.fillTypeToMixerWagonFillType[fillType];
    if mixerWagonFillType == nil then
        if self:allowFillType(fillType) and fillLevel < self.fillLevel and self.fillLevel > 0 then
            -- remove values from all fill types such that the ratio doesn't change
            fillLevel = Utils.clamp(fillLevel, 0, self:getCapacity());
            local delta = fillLevel - self.fillLevel;
            local newFillLevel = 0;
            for _, entry in pairs(self.mixerWagonFillTypes) do
                entry.fillLevel = math.max(entry.fillLevel + delta * entry.fillLevel / self.fillLevel, 0);
                newFillLevel = newFillLevel + entry.fillLevel;
            end
            self:raiseDirtyFlags(self.mixerWagonDirtyFlag);
            superFunc(self, newFillLevel, self.currentFillType, force, fillSourceStruct);
        end
        return;
    end
    local isSingleFilled = (self.fillLevel == mixerWagonFillType.fillLevel) and self:mixerWagonSuperAllowFillType(fillType);
    local limitedFillLevel = fillLevel;
    limitedFillLevel = math.min(limitedFillLevel, self:getCapacity() - self.fillLevel + mixerWagonFillType.fillLevel);
    -- dont check for validity any more. player can always empty a bit of the forage mixing at the feeding trough and fill up later
    --[[if not isSingleFilled then
        -- make sure that we are not above the max percentage
        limitedFillLevel = math.min(limitedFillLevel, mixerWagonFillType.maxPercentage * self:getCapacity());
        -- check that all other minPercentages can still be fullfilled
        local othersMinPercentagesFillLevel = 0;
        for _, entry in pairs(self.mixerWagonFillTypes) do
            if entry ~= mixerWagonFillType then
                othersMinPercentagesFillLevel = othersMinPercentagesFillLevel + entry.minPercentage * self:getCapacity()
            end
        end
        limitedFillLevel = math.min(limitedFillLevel, self:getCapacity()-othersMinPercentagesFillLevel);
    end]]
    limitedFillLevel = Utils.clamp(limitedFillLevel, 0, self:getCapacity());
    if limitedFillLevel ~= mixerWagonFillType.fillLevel then
        mixerWagonFillType.fillLevel = limitedFillLevel;
        self:raiseDirtyFlags(self.mixerWagonDirtyFlag);
    end
    -- calculate new fill level and forage is ok
    local newFillLevel = 0;
    local isForageOk = true;
    for _, entry in pairs(self.mixerWagonFillTypes) do
        newFillLevel = newFillLevel + entry.fillLevel;
    end
    for _, entry in pairs(self.mixerWagonFillTypes) do
        if entry.fillLevel < entry.minPercentage * newFillLevel-0.01 or entry.fillLevel > entry.maxPercentage * newFillLevel + 0.01 then
            isForageOk = false;
            break;
        end
    end
    local newFillType = Fillable.FILLTYPE_FORAGE_MIXING;
    if isForageOk then
        newFillType = Fillable.FILLTYPE_PIGFORAGE;
    else
        if isSingleFilled then
            newFillType = fillType;
        else
            -- maybe we are single filled now
            local singleFilledFillType;
            for _, entry in pairs(self.mixerWagonFillTypes) do
                if entry.fillLevel == newFillLevel then
                    isSingleFilled = true;
                    newFillType = next(entry.fillTypes);
                    break;
                end
            end
        end
    end
    superFunc(self, newFillLevel, newFillType, true, fillSourceStruct);
end

-- function MixerWagon:mixerWagonBaleTriggerCallback(triggerId, otherActorId, onEnter, onLeave, onStay, otherShapeId)
    -- if onEnter then
        -- -- this happens if a compound child of a deleted compound is entering
        -- if otherActorId ~= 0 then
            -- local object = g_currentMission:getNodeObject(otherActorId);
            -- if object ~= nil then
                -- if object:isa(Bale) then
                    -- local fillLevel = object:getFillLevel();
                    -- local fillType = object:getFillType();
                    -- if self:allowFillType(fillType, false) then
                        -- self:setFillLevel(self:getFillLevel(fillType)+fillLevel, fillType, false);
                    -- end
                    -- object:delete();
                    -- self.mixingActiveTimer = self.mixingActiveTimerMax;
                    -- self:raiseDirtyFlags(self.mixerWagonDirtyFlag);
                -- end;
            -- end;
        -- end;
    -- end;
-- end;

function MixerWagon:drawMixerWagonHelp(posY, param)
    -- do not draw mixer wagon help when in game message is visible
    if g_currentMission.inGameMessage:getIsVisible() then return; end;
    local posX = g_currentMission.hudHelpTextPosX1;
    local posX2 = g_currentMission.hudHelpTextPosX2;
    posY = posY - g_currentMission.hudHelpTextSize - 2*g_currentMission.hudHelpTextLineSpacing;
	setTextBold(true);
    renderText(posX, posY, g_currentMission.hudHelpTextSize, g_i18n:getText("mixing_ratio_pigForage"));
	setTextBold(false);
    for _, mixerWagonFillType in pairs(self.mixerWagonFillTypes) do
        posY = posY - self.fillTypeOverlayLineSpacing;
        local percentage = 0;
        if self.fillLevel > 0 then
            percentage = mixerWagonFillType.fillLevel / self.fillLevel;
        end
        if mixerWagonFillType.fillLevel > 0 then
            if self.currentFillType ~= Fillable.FILLTYPE_PIGFORAGE or (percentage >= mixerWagonFillType.minPercentage and percentage <= mixerWagonFillType.maxPercentage) then
                mixerWagonFillType.statusBar:setColor(unpack(mixerWagonFillType.statusBarColor));
            else
                mixerWagonFillType.statusBar:setColor(unpack(mixerWagonFillType.statusBarColor2));
            end
        end
        mixerWagonFillType.statusBar:setPosition(posX2 - mixerWagonFillType.statusBar.width, posY+self.mixerStatusbarOffsetY);
        mixerWagonFillType.statusBar:setValue(percentage);
        mixerWagonFillType.statusBar:render();
		renderText(posX, posY + (self.fillTypeOverlayLineSpacing-g_currentMission.hudHelpTextSize)/2, g_currentMission.hudHelpTextSize, mixerWagonFillType.nameI18N);
        setTextAlignment(RenderText.ALIGN_CENTER);
        renderText(mixerWagonFillType.statusBar.x + mixerWagonFillType.statusBar.width/2, mixerWagonFillType.statusBar.y + mixerWagonFillType.statusBar.height + self.mixerFillLevelTextOffsetY, getCorrectTextSize(0.018*g_currentMission.vehicleHudScale), string.format("%1.2f%%", percentage * 100));
        setTextAlignment(RenderText.ALIGN_LEFT);
        -- draw range markers
        local y = mixerWagonFillType.statusBar.y + mixerWagonFillType.statusBar.height /2 + (mixerWagonFillType.statusBar.height - self.mixerWagonFillTypeRangeMarkerRightOverlay.height)/2;
        local maxLeft = mixerWagonFillType.statusBar.x + mixerWagonFillType.statusBar.width;
        self.mixerWagonFillTypeRangeMarkerLeftOverlay.x = math.min(mixerWagonFillType.statusBar.x + mixerWagonFillType.statusBar.width*mixerWagonFillType.minPercentage, maxLeft);
        self.mixerWagonFillTypeRangeMarkerRightOverlay.x = math.min(mixerWagonFillType.statusBar.x + mixerWagonFillType.statusBar.width*mixerWagonFillType.maxPercentage, maxLeft);
        self.mixerWagonFillTypeRangeMarkerLeftOverlay.y = y;
        self.mixerWagonFillTypeRangeMarkerRightOverlay.y = y;
        self.mixerWagonFillTypeRangeMarkerLeftOverlay:render();
        self.mixerWagonFillTypeRangeMarkerRightOverlay:render();
   end
end

function MixerWagon:onTurnedOn(noEventSend)
    if self.isClient then
        Utils.stopSample(self.sampleMixerWagonStop);
        if self:getIsActiveForSound() then
            Utils.playSample(self.sampleMixerWagonStart, 1, 0, nil);
        end;
    end;
end;

function MixerWagon:onTurnedOff(noEventSend)
    if self.isClient then
        Utils.stopSample(self.sampleMixerWagonStart);
        if self:getIsActiveForSound() then
            Utils.playSample(self.sampleMixerWagonStop, 1, 0, nil);
        end;
    end;
end;

function MixerWagon:getCanBeTurnedOn(superFunc)
    if not self.mixerWagonNeedsTurnOn then
        return false;
    end;
    if superFunc ~= nil then
        return superFunc(self, isTurnedOn);
    end
    return true;    
end;

function MixerWagon:getIsTurnedOnAllowed(superFunc, isTurnedOn)
    if not self.mixerWagonNeedsTurnOn then
        return false;
    end;
    if superFunc ~= nil then
        return superFunc(self, isTurnedOn);
    end
    return true;
end;

function MixerWagon:getAllowFillShovel(superFunc, fillType)
    if self:getIsTurnedOn() then
        if superFunc ~= nil then
            return superFunc(self, fillType);
        end
    end
    return false;
end;

function MixerWagon:fillShovelFromGroundValue(superFunc, areas, fillType)
    local value = false;
    if superFunc ~= nil then
        value = superFunc(self, areas, fillType)
    end
    if value then
        self.mixerWagonLastPickupTime = g_currentMission.time;
    end
    return value;
end

function MixerWagon:fillShovelFromTrigger(superFunc, shovelTrigger, deltaFillLevel, fillType, dt)
    local value = 0;
    if superFunc ~= nil then
        value = superFunc(self, shovelTrigger, deltaFillLevel, fillType, dt)
    end
    if value > 0 then
        self.mixerWagonLastPickupTime = g_currentMission.time;
    end
    return value;
end